/* Derived from LinuxBIOS */

#include <lib.h>
#include <arch/io.h>

#define UART_BASE SERIAL_IOBASE
#define DIVISOR (115200/SERIAL_SPEED)

/* Data */
#define UART_RBR 0x00
#define UART_TBR 0x00

/* Control */
#define UART_IER 0x01
#define UART_IIR 0x02
#define UART_FCR 0x02
#define UART_LCR 0x03
#define UART_MCR 0x04
#define UART_DLL 0x00
#define UART_DLM 0x01

/* Status */
#define UART_LSR 0x05
#define UART_MSR 0x06
#define UART_SCR 0x07

#define LCS 0x3 /* N81 */

void serial_init(void)
{
    /* disable interrupt */
    outb(0, UART_BASE + UART_IER);
    /* enable fifo */
    outb(1, UART_BASE + UART_FCR);
    /* assert DTR and RTS so the other end is happy */
    outb(3, UART_BASE + UART_MCR);
    /* set baud rate divisor */
    outb(0x80 | LCS, UART_BASE + UART_LCR);
    outb(DIVISOR, UART_BASE + UART_DLL);
    outb(DIVISOR >> 8, UART_BASE + UART_DLM);
    outb(LCS, UART_BASE + UART_LCR);
}

void serial_putchar(unsigned char c)
{
    while ((inb(UART_BASE + UART_LSR) & 0x20) == 0)
	;
    outb(c, UART_BASE + UART_TBR);
}

int serial_havechar(void)
{
    return inb(UART_BASE + UART_LSR) & 1;
}

int serial_getchar(void)
{
    while (!serial_havechar())
	;
    return inb(UART_BASE + UART_RBR);
}
