/* Simple/stupid VGA text console driver.
 * This code does not do any initialization of VGA device.
 * It assumes the device is in normal 80x25 text mode (BIOS mode 3).
 * LinuxBIOS should have programmed the device before booting us.
 *
 * 2003-07 SONE Takeshi
 */

#include <lib.h>
#include <arch/io.h>

#define WIDTH 80
#define HEIGHT 25

struct vram_cell {
    unsigned char ch;
    unsigned char attr;
};
typedef struct vram_cell vram_row[WIDTH];
#define vram ((vram_row *) phys_to_virt(0xb8000))

int cursor_y, cursor_x;

#define WHITE 7

static void clear_line(int y)
{
    int x;

    for (x = 0; x < WIDTH; x++) {
        vram[y][x].ch = ' ';
        vram[y][x].attr = WHITE;
    }
}

static void scroll_up(void)
{
    int i;
    struct vram_cell *src, *dst; 

    /* move up */
    dst = vram[0];
    src = vram[1];
    for (i = 0; i < WIDTH * (HEIGHT-1); i++)
        dst[i] = src[i];

    cursor_y--;

    /* clear bottom line */
    clear_line(HEIGHT-1);
}

static inline void crtc_write(unsigned char data, unsigned char index)
{
    outb(index, 0x3d4);
    outb(data, 0x3d5);
}

static inline unsigned char crtc_read(unsigned char index)
{
    outb(index, 0x3d4);
    return inb(0x3d5);
}

static void update_cursor(void)
{
    unsigned int addr;

    addr = cursor_x + WIDTH*cursor_y;
    crtc_write(addr>>8, 0x0e);
    crtc_write(addr, 0x0f);
}

void vga_init(void)
{
    unsigned int cursor_addr;

    /* try to obtain cursor position from hardware */
    cursor_addr = ((unsigned int) crtc_read(0x0e)) << 8;
    cursor_addr += crtc_read(0x0f);
    cursor_x = cursor_addr % WIDTH;
    cursor_y = cursor_addr / WIDTH;
    if (cursor_y < HEIGHT) {
	/* looks sane, use this position, but start a new line first */
	if (cursor_x != 0) {
	    vga_putchar('\r');
	    vga_putchar('\n');
	}
    } else {
	/* can't get cursor position, use bottom line */
	cursor_y = HEIGHT;
	cursor_x = 0;
	scroll_up();
    }
    update_cursor();
}

void vga_putchar(unsigned char c)
{
    switch (c) {
    case '\r':
        cursor_x = 0;
        break;
    case '\n':
        cursor_y++;
        break;
    case '\b':
	cursor_x--;
	break;
    case '\t':
	cursor_x = (cursor_x + 8) & ~7;
	break;
    default:
        vram[cursor_y][cursor_x].ch = c;
        vram[cursor_y][cursor_x].attr = WHITE;
        cursor_x++;
        break;
    }
    while (cursor_x < 0) {
	if (cursor_y > 0) {
	    cursor_y--;
	    cursor_x += WIDTH;
	} else {
	    cursor_x = cursor_y = 0;
	}
    }
    if (cursor_x >= WIDTH) {
        cursor_x = 0;
        cursor_y++;
    }
    if (cursor_y >= HEIGHT)
        scroll_up();
    update_cursor();
}
