#ifndef LIB_H
#define LIB_H

#include <stddef.h>
#include <arch/stdint.h>

#ifdef VGA_CONSOLE
void vga_init(void);
void vga_putchar(unsigned char c);
#endif

#ifdef BTEXT_CONSOLE
void btext_init(void);
void btext_putchar(unsigned char c);
void map_boot_text(void);
#endif

#ifdef PC_KEYBOARD
int kbd_havekey(void);
int kbd_havechar(void);
int kbd_getchar(void);
#endif

#ifdef SERIAL_CONSOLE
void serial_init(void);
void serial_putchar(unsigned char c);
int serial_havechar(void);
int serial_getchar(void);
#endif 

void console_init(void);
int putchar(int c);
int puts(const char *s);
int printf(const char *fmt, ...) __attribute__ ((format (printf, 1, 2)));
int havekey(void);
int havechar(void);
int getchar(void);
int getline(char *buf, int max);

extern int last_putchar;

void *malloc(size_t size);
void free(void *ptr);
void *calloc(size_t nmemb, size_t size);
void *realloc(void *ptr, size_t size);
void malloc_check(void);
void malloc_diag(void);

size_t strnlen(const char *src, size_t max);
size_t strlen(const char *src);
int strcmp(const char *s1, const char *s2);
int memcmp(const void *s1, const void *s2, size_t n);
void *memcpy(void *dest, const void *src, size_t n);
void *memset(void *s, int c, size_t n);
void *memmove(void *dest, const void *src, size_t n);
int isspace(int c);
int isdigit(int c);
int tolower(int c);
char *strdup(const char *s);
char *strchr(const char *s, int c);
char *strncpy(char *to, const char *from, int count);
unsigned long long simple_strtoull(const char *cp,char **endp,unsigned int base);
unsigned long long strtoull_with_suffix(const char *cp,char **endp,unsigned int base);

unsigned int get_le32(const unsigned char *);
unsigned int get_le16(const unsigned char *);
void hexdump(const void *p, unsigned int len);

long long simple_strtoll(const char *cp,char **endp,unsigned int base);

void halt(void) __attribute__((noreturn));

#define abort() halt()

#define LOADER_NOT_SUPPORT 0xbadf11e

struct sys_info;
int elf_load(struct sys_info *, const char *filename, const char *cmdline);

#if LINUX_LOADER
int linux_load(struct sys_info *, const char *filename, const char *cmdline);
#else
#define linux_load(x,y,z) LOADER_NOT_SUPPORT /* nop */
#endif

#endif /* LIB_H */
