#ifndef SOUND_H
#define SOUND_H

#include <arch/stdint.h>
#include <pci.h>

/*
 * Application interface
 */
int sound_init(void);
void sound_set_rate(int rate);
void sound_set_volume(int volume);
int sound_write(const void *buf, int size);
void sound_start(void);
void sound_stop(void);
int sound_is_active(void);

/*
 * Driver interface
 */

struct sound_ops {
    int (*init)(struct pci_dev *);
    void (*set_rate)(int rate);
    void (*set_volume)(int volume);
    int (*write)(const void *buf, int size);
    int (*is_active)(void);
    void (*stop)(void);
};

struct sound_driver {
    uint16_t vendor;
    uint16_t device;
    struct sound_ops *ops;
};

#define __sound_driver __attribute__((unused, section(".rodata.sound_drivers")))

/* defined by the linker */
extern struct sound_driver sound_drivers_start[];
extern struct sound_driver sound_drivers_end[];

#endif /* SOUND_H */
