#include <pci.h>
#include <lib.h>
#define DEBUG_THIS DEBUG_PCI
#include <debug.h>

static struct pci_dev *dev_list;
static int n_devs;

static void pci_scan_bus(int bus)
{
    int devnum, fn, is_multi, vid;
    pci_addr addr;
    unsigned int htype;

    for (devnum = 0; devnum < 32; devnum++) {
	is_multi = 0;
	for (fn = 0; fn==0 || (is_multi && fn<8); fn++) {
	    addr = PCI_ADDR(bus, devnum, fn);
	    vid = pci_read16(addr, PCI_VENDOR_ID);
	    if (vid==0xffff || vid==0)
		continue;

	    htype = pci_read8(addr, PCI_HEADER_TYPE);
	    if (fn == 0)
		is_multi = htype & 0x80;
	    htype &= 0x7f;

	    if (dev_list) {
		dev_list[n_devs].addr = addr;
		dev_list[n_devs].vendor = vid;
		dev_list[n_devs].device = pci_read16(addr, PCI_DEVICE_ID);
		dev_list[n_devs].devclass = pci_read16(addr, PCI_CLASS_DEVICE);
		dev_list[n_devs].prog_if = pci_read8(addr, PCI_CLASS_PROG);
	    }
	    n_devs++;

	    if (htype==PCI_HEADER_TYPE_BRIDGE
		    || htype==PCI_HEADER_TYPE_CARDBUS)
		pci_scan_bus(pci_read8(addr, PCI_SECONDARY_BUS));
	}
    }
}

void pci_init(void)
{
    /* Count devices */
    dev_list = 0;
    debug("Scanning PCI: ");
    pci_scan_bus(0);
    pci_scan_bus(1);
    pci_scan_bus(3);
    debug("found %d devices\n", n_devs);

    /* Make the list */
    dev_list = malloc(n_devs * sizeof(struct pci_dev));
    n_devs = 0;
    pci_scan_bus(0);
    pci_scan_bus(1);
    pci_scan_bus(3);

#if DEBUG
    {
	int i;
	for (i = 0; i < n_devs; i++) {
	    debug("%02x:%02x.%x %04x:%04x %04x %02x\n", 
		    PCI_BUS(dev_list[i].addr),
		    PCI_DEV(dev_list[i].addr),
		    PCI_FN(dev_list[i].addr),
		    dev_list[i].vendor,
		    dev_list[i].device,
		    dev_list[i].devclass,
		    dev_list[i].prog_if);
	}
    }
#endif
}

struct pci_dev *pci_find_device(int vendor, int device, int devclass, int prog_if, int index)
{
    struct pci_dev *dev;

    for (dev = dev_list; dev < dev_list + n_devs; dev++) {
	if (vendor < 0 || vendor==dev->vendor)
	    if (device < 0 || device==dev->device)
		if (devclass < 0 || devclass==dev->devclass)
		    if (prog_if < 0 || prog_if==dev->prog_if) {
			if (index == 0)
			    return dev;
			index--;
		    }
    }
    return NULL;
}
