#include <sound.h>
#include <pci.h>

static struct sound_ops *ops;

int sound_init(void)
{
    struct sound_driver *drv;
    struct pci_dev *dev;

    for (drv = sound_drivers_start; drv < sound_drivers_end; drv++) {
	dev = pci_find_device(drv->vendor, drv->device, -1, -1, 0);
	if (dev) {
	    if (drv->ops->init(dev) == 0) {
		ops = drv->ops;
		return 0;
	    }
	}
    }
    printf("No sound device found\n");
    return -1;
}

void sound_set_rate(int rate)
{
    if (ops && ops->set_rate)
	ops->set_rate(rate);
}

void sound_set_volume(int volume)
{
    if (ops && ops->set_volume)
	ops->set_volume(volume);
}

int sound_write(const void *buf, int size)
{
    if (ops && ops->write)
	return ops->write(buf, size);
    return -1;
}

int sound_is_active(void)
{
    if (ops && ops->is_active)
	return ops->is_active();
    return 0;
}

void sound_stop(void)
{
    if (ops && ops->stop)
	ops->stop();
}
